﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Data;
using System.Globalization;
using System.Reflection;
using System.Text;
using System.Transactions;
using Produmex.Foundation;
using Produmex.Foundation.Barcode;
using Produmex.Foundation.Data;
using Produmex.Foundation.Data.Sbo;
using Produmex.Foundation.Data.Sbo.BusinessObjects;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Convertors;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions.Tables;
using Produmex.Foundation.Data.Sbo.DataObjects;
using Produmex.Foundation.Data.Sbo.Providers;
using Produmex.Foundation.Data.Sbo.Utilities;
using Produmex.Foundation.Data.SqlClient;
using Produmex.Foundation.Diagnostics;
using Produmex.Foundation.GS1;
using Produmex.Foundation.Messages;
using Produmex.Foundation.SlimScreen;
using Produmex.Foundation.SlimScreen.Interfaces;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions.DataObjects;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions.KnownDataSets;
using Produmex.Foundation.SlimScreen.WinGui;
using Produmex.Foundation.Utilities;
using Produmex.Foundation.Workflows;
using Produmex.Foundation.Workflows.Parameters;
using Produmex.Foundation.Wwf.Sbo;
using Produmex.Foundation.Wwf.Sbo.DataObjects;
using Produmex.Foundation.Wwf.Sbo.LocalServices;
using Produmex.Sbo.Logex.Data.BusinessObjects;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables;
using Produmex.Sbo.Logex.Data.DataObjects;
using Produmex.Sbo.Logex.Data.Devices;
using Produmex.Sbo.Logex.Data.Generators.BatchNumberGenerators;
using Produmex.Sbo.Logex.Data.Generators.PackingController;
using Produmex.Sbo.Logex.Data.Providers;
using Produmex.Sbo.Logex.Data.ViewObjects.Definitions.Views;

// PmxWorkflowExecutionTypes.HOOK_FLOW
namespace Produmex.Sbo.Logex.WorkflowScripts
{
	public class WorkflowScript_AfterItemPickedHookScript : WorkflowInstanceScriptBase
	{

		private static class Definitions
		{

			public const string REPORT_PATH_ITEM_LABEL = @"DefaultItemLabel.rpt";
			public const int NUMBER_OF_LABELS = 1;

		}

		private static readonly ILog s_log = LogProvider.GetLogger(MethodInfo.GetCurrentMethod().DeclaringType);

		// Input parameters * do not change *
		public ReadOnlyBinder<CultureInfo> DefaultCultureInfo;
		public ReadOnlyBinder<PmxOseCompany> PmxOseCompany;

		public ReadOnlyBinder<Collection<PickingItemInfo>> PickedItemInfos;
		public ReadOnlyBinder<int?> LUID;
		public ReadOnlyBinder<Collection<string>> ListOfSerialNumbers;
		public ReadOnlyBinder<int> DocEntry;
		public ReadOnlyBinder<string> ObjType;

		// Sub-flows
		// <none>


		public WorkflowScript_AfterItemPickedHookScript(WorkflowInstanceBase parent, WorkflowInstanceFactory factory)
			: base(parent, factory)
		{
		}

		#region WorkflowInstanceScriptBase Members

		protected override void Execute()
		{
			// Parameters in scope
			Session session = GetScopeParameter("Session") as Session;
			ISboProviderService sboProviderService = GetScopeParameter("<WwfService>ISboService") as ISboProviderService;

			double numberOfItemLabels = Definitions.NUMBER_OF_LABELS;
			Message msg = null;
			string initialErrorKey = null;


			Step_AskAmount:

			session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IEnterQuantityScreen),
				DefaultCultureInfo.Get(), BuildParamCollection(
			   "InitialErrorKey", initialErrorKey,
			   "TitleKey", "MSG_TITLE_ENTER_QUANTITY_ITEM_LABELS",
			   "InitialQuantity", numberOfItemLabels,
			   "NumberOfDigitsAfterDecimal", 0,
			   "MaximumQuantity", -1,			 
			   "SetFocusToQuantityControl", true
			   ));
			msg = WaitForMessage();

			initialErrorKey = null;

			if (msg.Name.EndsWith(".QuantityEntered"))
			{
				numberOfItemLabels = ExtractParameter<double>(msg.Parameters, "quantity");
				goto Step_PrintReport;
			}
			else
			{
				return;
			}



			Step_PrintReport:

			#region PRINT REPORT
			// Print LABEL
			string reportPathItemLabel = Definitions.REPORT_PATH_ITEM_LABEL;

			try
			{
				
				string itemCode = PickedItemInfos.Get()[0].ItemCode;
				string batchNumber = PickedItemInfos.Get()[0].BatchNumber;
				double quantity = PickedItemInfos.Get()[0].Quantity;

				sboProviderService.InvokeMethodWithDbConnection<PmxOsePrinter>(false, false, null, null, new DelegateWithPmxDbConnection<PmxOsePrinter>(ExecutePrintItemLabel), itemCode, batchNumber, quantity,Convert.ToInt32(numberOfItemLabels),session.DeviceInstanceId, reportPathItemLabel);

			}
			catch (Exception ex)
			{
				s_log.Error(ex.ToString());
				throw ex;
			}

			#endregion


		}

		#endregion

		#region HELPERS

		/// <summary>
		/// Executes the print label.
		/// </summary>
		/// <param name="conn">The conn.</param>
		/// <param name="parameters">The parameters.</param>
		/// <returns></returns>
		private static PmxOsePrinter ExecutePrintItemLabel(PmxDbConnection conn, object[] parameters)
		{
			PmxPrintReportProvider prov = new PmxPrintReportProvider(conn);

			Collection<ReportParameter> reportParameters = new Collection<ReportParameter>();
			reportParameters.Add(new ReportParameter("@itemCode", (string)parameters[0]));
			reportParameters.Add(new ReportParameter("@batchNumber", (string)parameters[1]));
			reportParameters.Add(new ReportParameter("@serialNumber", string.Empty));
			reportParameters.Add(new ReportParameter("@bestBeforeDate", null));
			reportParameters.Add(new ReportParameter("@docType", string.Empty));
			reportParameters.Add(new ReportParameter("@docEntry", null));
			reportParameters.Add(new ReportParameter("@lineNum", null));
			reportParameters.Add(new ReportParameter("@batchNumber2", string.Empty));
			reportParameters.Add(new ReportParameter("@quantity", (double)parameters[2]));
			reportParameters.Add(new ReportParameter("@uom", string.Empty));
			reportParameters.Add(new ReportParameter("@weight", null));
			reportParameters.Add(new ReportParameter("@weightUom", string.Empty));
			reportParameters.Add(new ReportParameter("@packagingTypeCode", string.Empty));

			return prov.PrintReport(reportParameters, (string)parameters[5], (int)parameters[3], (string)parameters[4], null, false);
		}

		#endregion


	}
}