using System.Globalization;
using System.Reflection;
using Produmex.Foundation.Data.Sbo.DataObjects;
using Produmex.Foundation.Diagnostics;
using Produmex.Foundation.Workflows;
using Produmex.Foundation.Workflows.Parameters;
using Produmex.Sbo.Logex.Data.BusinessObjects;
using System.Collections.Generic;
using Produmex.Foundation.Collections;
using System.Collections.ObjectModel;
using Produmex.Foundation.Wwf.Sbo.LocalServices;
using Produmex.Foundation.SlimScreen;
using Produmex.Foundation.Data.Sbo;
using Produmex.Foundation.Data.Sbo.Utilities;

// PmxWorkflowExecutionTypes.HOOK_FLOW
namespace WorkflowScript_AfterLogisticUnitPackedHookScript
{
    /// <summary>
    /// This is a hook script, meant to be customized on a "per project" basis
    /// </summary>
    public class WorkflowScript_AfterLogisticUnitPackedHookScript : WorkflowInstanceScriptBase
    {
        private static readonly ILog s_log = LogProvider.GetLogger(MethodInfo.GetCurrentMethod().DeclaringType);
		private static class Definitions
		{

			public const string DATABASE = "PMX_SBODemo_US_191";
			public const string STOREDPROCEDURE = "PMX_GSE";

		}


		// Input parameters * do not change *
		public ReadOnlyBinder<CultureInfo> DefaultCultureInfo;
        public ReadOnlyBinder<PmxOseCompany> PmxOseCompany;
        public ReadOnlyBinder<List<int>> LUIDs;
        public ReadOnlyBinder<Collection<int>> ListOfPickListEntries;
        public ReadOnlyBinder<int?> WaveKey;
        public ReadOnlyBinder<int?> RouteKey;

        // Output parameters * do not change *
        public ReadWriteBinder<PmxDictionary<string, object>> DeliveryData;

        // Sub-flows
        // <none>

        /// <summary>
        /// Gets the current list of pick list entries.
        /// </summary>
        /// <value>The current list of pick list entries.</value>
        private Collection<int> currentListOfPickListEntries
        {
            get
            {
                return ListOfPickListEntries == null ? null : ListOfPickListEntries.Get();
            }
        }

        /// <summary>
        /// Gets the current wave key.
        /// </summary>
        /// <value>The current wave key.</value>
        private int? currentWaveKey
        {
            get
            {
                return WaveKey == null ? null : WaveKey.Get();
            }
        }

        /// <summary>
        /// Gets the current route key.
        /// </summary>
        /// <value>The current route key.</value>
        private int? currentRouteKey
        {
            get
            {
                return RouteKey == null ? null : RouteKey.Get();
            }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="WorkflowScript_AfterLogisticUnitPackedHookScript"/> class.
        /// </summary>
        /// <param name="parent">The parent.</param>
        /// <param name="factory">The factory.</param>
        public WorkflowScript_AfterLogisticUnitPackedHookScript( WorkflowInstanceBase parent, WorkflowInstanceFactory factory )
            : base(parent, factory)
        {
        }

        #region WorkflowInstanceScriptBase Members

        protected override void Execute()
        {
			// Parameters in scope
			Session session = GetScopeParameter("Session") as Session;
			ISboProviderService sboProviderService = GetScopeParameter("<WwfService>ISboService") as ISboProviderService;


		
			//execute store procedure for each luid
			foreach (int luid in LUIDs.Get())
			{

				s_log.Warn($"End calling procedure (lUID = {luid}");

				string query = string.Format("Exec {0}.dbo.{1} {2}", Definitions.DATABASE, Definitions.STOREDPROCEDURE, luid.ToString() );

				sboProviderService.InvokeMethodWithDbConnection<object>(false, false, null, null, new DelegateWithPmxDbConnection<object>(
				delegate (PmxDbConnection conn, object[] parameters)
				{				
					SboRecordsetHelper.RunQueryNoReturn(s_log, query, conn);
					return null;
				}));

				s_log.Warn($"End calling procedure");

			}

		

		}

        #endregion
    }
}
